define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/string",
        "dijit/_Widget",        
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",
        "dojo/text!./templates/LDAPServerInfoWizardPage.html",
        "dojo/i18n!../nls/user_resources",
        "dojo/dom-style",
        "dojo/request/xhr",
        "dojo/json",
        "./BusyDialog",
        "dijit/form/RadioButton",
        "dojox/form/Uploader",
        "dojox/form/uploader/plugins/IFrame",
        "idx/form/TextBox",
        "idx/form/NumberTextBox",
        "idx/form/RadioButtonSet",
        "dijit/form/Button"
        ],
        function(dDeclare,
                dLang,
                dString,
                dWidget,
                dTemplatedMixin,
                dWidgetsInTemplateMixin,
                templateText,
                resources,
                domStyle,
                xhr,
                JSON,
                BusyDialog) {
return dDeclare("admin.users.dialogs.LDAPServerInfoWizardPage", [dWidget,dTemplatedMixin,dWidgetsInTemplateMixin], {

	/**
     * The widget template for the dijit._TemplatedMixin base class.
     */
    templateString: templateText,
    
    /** 
     * Indicate if this widget is also a container
     * @private
     */
    isContainer: false,
    
    /**
     * Indicate if there are widgets in the template
     */
    widgetsInTemplate: true,
    
    fieldWidth: 150,
    
    pageDescription: resources["ldapInfoDescription"],
    
    hostLabel: resources["hostLabel"],
    
    portLabel: resources["portLabel"],
    
    encryptionMethodLabel: resources["encryptionMethodLabel"],
    
    sslEncryption: resources["sslEncryption"],
    
    noEncryption: resources["noEncryption"],
    
    authenticationMethodLabel: resources["authenticationMethodLabel"],
    
    simpleAuthentication: resources["simpleAuthentication"],
    
    noAuthentication: resources["noAuthentication"],
    
    userLabel:  resources["userDNLabel"],
    
    passwordLabel: resources["passwordLabel"],
    
    checkNetworkParameterButtonLabel: resources["checkNetworkButtonLabel"],
    
    testConnectionSuccess: resources["testConnectionSuccess"],
    
    testConnectionFailed: resources["testConnectionFailed"],
    
    busyDialog:null,
    
    /**
	 * Get the description of this page
	 */
	getDescription:function() {
		return this.pageDescription;
	},
    
	
	/**
	 * Check if this page is complete
	 */
	isComplete:function() {
		if (!this.portNode.isValid()) {
			return false;
		}
		var host = dLang.trim(this.hostNode.get("value"));
		var port = this.portNode.get("value");
		console.log(host, port);
		if (host == "" || isNaN(port)) {
			return false;
		}
		
		var authMethod = this.authenticationNode.get("value");
		if (authMethod == "simple") {
			var user = dLang.trim(this.userNode.get("value"));
			var password = dLang.trim(this.passwordNode.get("value"));
			if (user == "" || password == "") {
				return false;
			}
		}
		return true;
	},
	
	/**
     * The destroy function
     */
    destroy: function() {
    	this.inherited(arguments);
    	if (this.busyDialog != null) {
    		this.busyDialog.close();
    	}
    },
	
    /**
     * The startup function
     */
	startup:function() {
		this.inherited(arguments);
		this.hostNode.on("Change", dLang.hitch(this, "_validate"));
		this.portNode.on("Change", dLang.hitch(this, "_validate"));
		this.encryptionNode.on("Change", dLang.hitch(this, "_validate"));
		this.authenticationNode.on("Change", dLang.hitch(this, "_validate"));
		this.userNode.on("Change", dLang.hitch(this, "_validate"));
		this.passwordNode.on("Change", dLang.hitch(this, "_validate"));
		this.hostNode.set('intermediateChanges', true);
		this.portNode.set('intermediateChanges', true);
		this.encryptionNode.set('intermediateChanges', true);
		this.userNode.set('intermediateChanges', true);
		this.passwordNode.set('intermediateChanges', true);
		this.testButton.on("click", dLang.hitch(this, "_testLDAPInfo"));
		//override the formatter to remove the default number formatting
		this.portNode._formatter = function(value) {return value;};
		console.log("startup complete.")
	},
	
	/**
	 * Initialize this wizard page with existing LDAP info
	 */
	init:function() {
		if (this.controller.ldapInfo != null) {
			console.log("initilizing default ldap information.");
			var ldapInfo = this.controller.ldapInfo;
			var host = ldapInfo.singleValueProperties["com.ibm.nex.core.models.policy.ldapHost"];
			var port = ldapInfo.singleValueProperties["com.ibm.nex.core.models.policy.ldapPort"];
			var ssl = ldapInfo.singleValueProperties["com.ibm.nex.core.models.policy.isSSLEnabled"];
			var securityPrincipal = ldapInfo.singleValueProperties["com.ibm.nex.core.models.policy.ldapSecurityPrincipal"];
			var password = ldapInfo.singleValueProperties["com.ibm.nex.core.models.policy.ldapPassword"];
			var securityMethod = ldapInfo.singleValueProperties["com.ibm.nex.core.models.policy.ldapSecurityAuthentication"];
			this.hostNode.set("value", host);
			this.portNode.set("value", port);
			this.userNode.set("value", securityPrincipal);
			this.passwordNode.set("value", password);
			this.authenticationNode.set("value", securityMethod);
			if (ssl == "t" || ssl == "T") {
				this.encryptionNode.set("value", "ssl");
			}
		}
	},
	
	/**
	 * Get the ldap information on this page
	 */
	getLDAPInfo:function() {
		var ldapInfo = {'host': this.hostNode.get("value"), 
				'port': this.portNode.get("value"),
				'securityType': this.authenticationNode.get("value"),
				'ssl': this.encryptionNode.get("value") == "ssl",
				'securityPrincipal': this.userNode.get("value"),
				'password': this.passwordNode.get("value")};
		return ldapInfo;
	},
	
	/**
	 * Invoke test connection
	 */
	_testLDAPInfo:function() {
		var ldapInfo = {'host': this.hostNode.get("value"), 
						'port': this.portNode.get("value"),
						'securityType': this.authenticationNode.get("value"),
						'ssl': this.encryptionNode.get("value") == "ssl",
						'securityPrincipal': this.userNode.get("value"),
						'password': this.passwordNode.get("value")};
		if (this.busyDialog == null) {
			this.busyDialog = new BusyDialog({title:"Check Network Parameters"});
			this.busyDialog.startup();
		}
	    this.busyDialog.setMessage("Checking network parameters...");
		this.busyDialog.show();
		this.busyDialog.setClosable(false);
		xhr.post("/optim/rest/security/registry/ldaptest", 
	        	{
	        	handleAs: "json",
	        	data: JSON.stringify(ldapInfo),
	        }).then(dLang.hitch(this, "_testComplete"), dLang.hitch(this, "_testFailed"), function(evt){
	        	//do nothing
	        	console.log(evt);
	        });
	},
	
	/**
	 * Test connection successful
	 */
	_testComplete:function(data) {
		console.log("_testComplete:", data);
		this.busyDialog.setSuccessMessage(this.testConnectionSuccess);
		this.busyDialog.setClosable(true);
		//this.busyDialog._hide();
	},
	
	/**
	 * Test connection failed
	 */
	_testFailed:function(data) {
		console.log("_testFailed: ", data);
		this.busyDialog.setErrorMessage(this.testConnectionFailed);
		this.busyDialog.setClosable(true);
	},
	
	/**
	 * Input values changed - update but states
	 */
	_validate:function() {
		console.log("_validate", arguments);
		var host = this.hostNode.get("value");
		var port = this.portNode.get("value");
		var authMethod = this.authenticationNode.get("value");
		var user = this.userNode.get("value");
		var password = this.passwordNode.get("value");
		var valid = this.portNode.isValid();
		if (host == "" || port == "" || isNaN(port)) {
			valid = false;
		}
		if (authMethod == "simple") {
			this.userNode.set('disabled', false);
			this.passwordNode.set('disabled', false);
		}
		else {
			this.userNode.set('disabled', true);
			this.passwordNode.set('disabled', true);
		}
		if (authMethod == "simple" && (password == "" || user == "")) {
			valid = false;
		}
		console.log(host, port, authMethod, user, password);
		this.testButton.set("disabled", !valid);
		if (this.controller != null) {
			this.controller.updateWizardButtonStates();
		}
	}
});
});
